
/* Simple logger.  Use `ipchains -o' to make this process log packets
 * to stdout */

static void dump_packet_ip(const struct iphdr *ip, 
			   const char *ifname,
			__u16 src_port, 
			__u16 dst_port,
			const char *proto)
{
	__u32 *opt = (__u32 *) (ip + 1);
	int opti;
	
	printf("%s %s %ld.%ld.%ld.%ld:%hu %ld.%ld.%ld.%ld:%hu"
	       " L=%hu S=0x%2.2hX I=%hu F=0x%4.4hX T=%hu",
	       ifname, proto,
	       (ntohl(ip->saddr)>>24)&0xFF,
	       (ntohl(ip->saddr)>>16)&0xFF,
	       (ntohl(ip->saddr)>>8)&0xFF,
	       (ntohl(ip->saddr))&0xFF,
	       src_port,
	       (ntohl(ip->daddr)>>24)&0xFF,
	       (ntohl(ip->daddr)>>16)&0xFF,
	       (ntohl(ip->daddr)>>8)&0xFF,
	       (ntohl(ip->daddr))&0xFF,
	       dst_port,
	       ntohs(ip->tot_len), ip->tos, ntohs(ip->id),
	       ntohs(ip->frag_off), ip->ttl);

	for (opti = 0; opti < (ip->ihl - sizeof(struct iphdr) / 4); opti++)
		printf(" O=0x%8.8X", *opt++);
	printf("\n");
}

static const struct iphdr *
readpkt_tcp(__u32 rulenum __attribute__((unused)), 
	const char *interface,
	const ip_chainlabel name __attribute__((unused)),
	__u16 packetsize,
	struct iphdr *hdr)
{
	struct tcphdr		*tcp=(struct tcphdr *)((__u32 *)ip+ip->ihl);

	dump_packet(hdr, interface, ntohs(tcp->source), ntohs(tcp->dest), 
	            "TCP");
	return NULL;
}

void cleanup_and_exit(int signal __attribute((unused)))
{
    fw_close_listen(0);
    fprintf(stderr, "Packets done = %u.\n", packets_done);
    exit(0);
}

int main(int argc, const char *argv[])
{
    struct ip_fw fw 
	= { { 0 }, { 0 }, { 0 }, { 0 }, /* SRC & DST = any */
	    0,				/* Filled in by fw_reg... */
	    IPPROTO_TCP,		/* Only TCP packets */
	    0,				/* Nothing special */
	    0,				/* Nothing special */
	    { 0, 0xFFFF },              /* SRCPT = any */
	    { 0, 0xFFFF },		/* DSTPT = any */
	    0,				/* redirpt = N/A */
	    65535,                      /* outputsize = all */
	    "",				/* interface = any */
	    0xFF, 0x00			/* TOS = unchanged */
	};
    int ret;

    /* Cleanup on receipt of ^C */
    signal(SIGINT, &cleanup_and_exit);

    if (!fw_init_listen(1, 0)) exit(1);

    if (!fw_register_interest(&fw, "input", readpkt_tcp, "ACCEPT", 1, 0)) 
            exit(1);

    ret = fw_do_listen(0);
    fw_close_listen(0);
    exit(!ret);
}
